#=============================================================================================
# The utiliy functions for the SGD module:
# These functions render various types of plots generated by the SGD module
# The dependencies for functionalities currently in use:
# ggplot2, GGally, reshape
# The dependencies for functionalities not currently used:
# cluster, mclust
#=============================================================================================



# ggbiplot script
# @Author: Vince Q. Vu: vqv@stat.osu.edu
# This function generates PCA biplots
ggbiplot <- function (pcobj, choices = 1:2, scale = 1, pc.biplot = TRUE, 
                      obs.scale = 1 - scale, var.scale = scale, groups = NULL, 
                      ellipse = FALSE, ellipse.prob = 0.68, labels = NULL, labels.size = 3, 
                      alpha = 1, var.axes = TRUE, circle = FALSE, circle.prob = 0.69, 
                      varname.size = 3, varname.adjust = 1.5, varname.abbrev = FALSE, 
                      ...) 
{
  library(ggplot2)
  library(plyr)
  library(scales)
  library(grid)
  stopifnot(length(choices) == 2)
  if (inherits(pcobj, "prcomp")) {
    nobs.factor <- sqrt(nrow(pcobj$x) - 1)
    d <- pcobj$sdev
    u <- sweep(pcobj$x, 2, 1/(d * nobs.factor), FUN = "*")
    v <- pcobj$rotation
  }
  else if (inherits(pcobj, "princomp")) {
    nobs.factor <- sqrt(pcobj$n.obs)
    d <- pcobj$sdev
    u <- sweep(pcobj$scores, 2, 1/(d * nobs.factor), FUN = "*")
    v <- pcobj$loadings
  }
  else if (inherits(pcobj, "PCA")) {
    nobs.factor <- sqrt(nrow(pcobj$call$X))
    d <- unlist(sqrt(pcobj$eig)[1])
    u <- sweep(pcobj$ind$coord, 2, 1/(d * nobs.factor), FUN = "*")
    v <- sweep(pcobj$var$coord, 2, sqrt(pcobj$eig[1:ncol(pcobj$var$coord), 
                                                  1]), FUN = "/")
  }
  else if (inherits(pcobj, "lda")) {
    nobs.factor <- sqrt(pcobj$N)
    d <- pcobj$svd
    u <- predict(pcobj)$x/nobs.factor
    v <- pcobj$scaling
    d.total <- sum(d^2)
  }
  else {
    stop("Expected a object of class prcomp, princomp, PCA, or lda")
  }
  choices <- pmin(choices, ncol(u))
  df.u <- as.data.frame(sweep(u[, choices], 2, d[choices]^obs.scale, 
                              FUN = "*"))
  v <- sweep(v, 2, d^var.scale, FUN = "*")
  df.v <- as.data.frame(v[, choices])
  names(df.u) <- c("xvar", "yvar")
  names(df.v) <- names(df.u)
  if (pc.biplot) {
    df.u <- df.u * nobs.factor
  }
  r <- sqrt(qchisq(circle.prob, df = 2)) * prod(colMeans(df.u^2))^(1/4)
  v.scale <- rowSums(v^2)
  df.v <- r * df.v/sqrt(max(v.scale))
  if (obs.scale == 0) {
    u.axis.labs <- paste("standardized PC", choices, sep = "")
  }
  else {
    u.axis.labs <- paste("PC", choices, sep = "")
  }
  u.axis.labs <- paste(u.axis.labs, sprintf("(%0.1f%% explained var.)", 
                                            100 * pcobj$sdev[choices]^2/sum(pcobj$sdev^2)))
  if (!is.null(labels)) {
    df.u$labels <- labels
  }
  if (!is.null(groups)) {
    df.u$groups <- groups
  }
  if (varname.abbrev) {
    df.v$varname <- abbreviate(rownames(v))
  }
  else {
    df.v$varname <- rownames(v)
  }
  df.v$angle <- with(df.v, (180/pi) * atan(yvar/xvar))
  df.v$hjust = with(df.v, (1 - varname.adjust * sign(xvar))/2)
  g <- ggplot(data = df.u, aes(x = xvar, y = yvar)) + xlab(u.axis.labs[1]) + 
    ylab(u.axis.labs[2]) + coord_equal()
  if (var.axes) {
    if (circle) {
      theta <- c(seq(-pi, pi, length = 50), seq(pi, -pi, 
                                                length = 50))
      circle <- data.frame(xvar = r * cos(theta), yvar = r * 
                             sin(theta))
      g <- g + geom_path(data = circle, color = muted("white"), 
                         size = 1/2, alpha = 1/3)
    }
    g <- g + geom_segment(data = df.v, aes(x = 0, y = 0, 
                                           xend = xvar, yend = yvar), arrow = arrow(length = unit(1/2, 
                                                                                                  "picas")), color = muted("red"))
  }
  if (!is.null(df.u$labels)) {
    if (!is.null(df.u$groups)) {
      g <- g + geom_text(aes(label = labels, color = groups), 
                         size = labels.size)
    }
    else {
      g <- g + geom_text(aes(label = labels), size = labels.size)
    }
  }
  else {
    if (!is.null(df.u$groups)) {
      g <- g + geom_point(aes(color = groups), alpha = alpha)
    }
    else {
      g <- g + geom_point(alpha = alpha)
    }
  }
  if (!is.null(df.u$groups) && ellipse) {
    theta <- c(seq(-pi, pi, length = 50), seq(pi, -pi, length = 50))
    circle <- cbind(cos(theta), sin(theta))
    ell <- ddply(df.u, "groups", function(x) {
      if (nrow(x) <= 2) {
        return(NULL)
      }
      sigma <- var(cbind(x$xvar, x$yvar))
      mu <- c(mean(x$xvar), mean(x$yvar))
      ed <- sqrt(qchisq(ellipse.prob, df = 2))
      data.frame(sweep(circle %*% chol(sigma) * ed, 2, 
                       mu, FUN = "+"), groups = x$groups[1])
    })
    names(ell)[1:2] <- c("xvar", "yvar")
    g <- g + geom_path(data = ell, aes(color = groups, group = groups))
  }
  if (var.axes) {
    g <- g + geom_text(data = df.v, aes(label = varname, 
                                        x = xvar, y = yvar, angle = angle, hjust = hjust), 
                       color = "darkred", size = varname.size)
  }
  return(g)
}


# This function generates violin plots
violin.by.gene <- function(normalized.dat, gene, smp.annot = NULL,smp.annot.name =NULL){
  require(ggplot2)
  
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(!gene %in% colnames(normalized.dat))
    stop("gene needs to be in colnames(normalized.dat)")
  
  if(is.null(smp.annot))
    smp.annot <- as.factor(rep("All observations",nrow(normalized.dat)))
  
  if(is.numeric(smp.annot))
    smp.annot <- factor(paste("Group",smp.annot))
  
  df <- data.frame(smp.annot = smp.annot, y=normalized.dat[,gene])
  
  g <- ggplot(aes(y = y, x = smp.annot), data = df) + 
    geom_violin(fill = "grey90", color = "white") +  
    geom_point(colour = "chartreuse3") + 
    ggtitle(paste("Expression of",gene)) +xlab(NULL)+ ylab("Expression level") + 
    geom_boxplot(width = .2,alpha=0)+
    theme_bw()
  if(nlevels(smp.annot)>1)
    g <- g + xlab(smp.annot.name)
  #theme(axis.text.x  = element_text(angle=60, vjust=0.5, size=10))
  return(g)
}


# This function generates scatter plots
scatter.by.gene <- function(normalized.dat, gene, smp.annot, smp.annot.name){
  require(ggplot2)
  
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(!gene %in% colnames(normalized.dat))
    stop("gene needs to be in colnames(normalized.dat)")
  
  if(!is.numeric(smp.annot))
    stop("smp.annot needs to be numeric")
  
  df <- data.frame(smp.annot = smp.annot, y=normalized.dat[,gene])
  
  g <- ggplot(aes(y = y, x = smp.annot), data = df) +
    stat_smooth(method = "lm",se=T, colour="black",linetype ="dashed")+
    geom_point(colour = "chartreuse3",size =4) + 
    ggtitle(paste("Expression of",gene)) + 
    xlab(smp.annot.name) + ylab("Expression level") + 
    theme_bw()
  return(g)
}



# Similarity plots:
# Finds top genes that co-vary with the gene of interest. 
# 1- Generates density plot for the gene of interest
# 2- Generates pair-wise scatter plot among these similar genes and report the pearson cor
# 3- If sample annotation is provided, it reports the correlations and colorcode the plots by covariate 
# @normalized.dat: normalized data matrix
# @gene: gene of interest
# @smp.annot: Categorical variable describing the sample. This needs to be as factor and have length equal to nrow(normalized.dat)
# @genes.to.compare.to: genes to compare to selected gene. If not null num.pairs.to.compare is ignored
# @num.pairs.to.compare: number of pairs of genes to compare the gene of intrest against. If genes.to.compare.to is not null, this is ignored

corplot.by.gene <- function(normalized.dat, gene, smp.annot = NULL,
                            #col.annot = NULL,
                            annot.name,
                            genes.to.compare.to = NULL, num.pairs.to.compare = 5){
  require(GGally)
  
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(!gene %in% colnames(normalized.dat))
    stop("gene needs to be in colnames(normalized.dat)")
  
  if(is.null(smp.annot))
    smp.annot <- as.factor(rep("all obs",nrow(normalized.dat)))
  
  if(is.numeric(smp.annot))
    smp.annot <- factor(paste("Group",smp.annot))
  
  if(!is.factor(smp.annot) | length(smp.annot)!=nrow(normalized.dat))
    stop("smp.annot needs to be factor of length nrow(normalized.dat)")
  
  if(!is.null(genes.to.compare.to)){
    if(any(!genes.to.compare.to %in% colnames(normalized.dat)))
      stop("genes.to.compare.to needs to be in colnames(normalized.dat)")
    genes.to.compare.to <- genes.to.compare.to[genes.to.compare.to !=gene]
    num.pairs.to.compare <- length(genes.to.compare.to)
    ttl <- paste("Expression distribution and covariation of",gene,"and selected",num.pairs.to.compare,"genes to compare to") 
  }else{
    cor.mat <- cor(normalized.dat) 
    top.correlated.genes <- colnames(cor.mat)[order(abs(cor.mat[gene,]),decreasing = T)[1 + 1:num.pairs.to.compare]]
    genes.to.compare.to <- top.correlated.genes
    ttl <- paste("Expression distribution and covariation of",gene,"and its top",num.pairs.to.compare,"correlated genes") 
  }
  
  if( num.pairs.to.compare<1 | num.pairs.to.compare > ncol(normalized.dat))
    stop("1<=num.paris.to.compare<=ncol(normalized.dat) needs to hold")
  
  
  df<- data.frame(smp.annot = abbreviate(smp.annot,minlength = 9),normalized.dat[,c(gene,genes.to.compare.to)])
  
  if(nlevels(as.factor(df$smp.annot))>1){
    group.col <- "smp.annot"
    #     if(!is.null(col.annot))
    #       group.col <- col.annot
    gp <- ggpairs(data = df,columns = 2:ncol(df),title = paste(ttl,"color-coded by",annot.name),colour = group.col)
  }else{
    gp <- ggpairs(data = df,columns = 2:ncol(df),title = ttl)
  }
  gp <- gp + theme_bw()
  gp <- gp + theme(legend.position = "none", 
                   panel.grid.major = element_blank(), 
                   axis.ticks = element_blank(), 
                   panel.border = element_rect( colour = "black", fill = NA))
  return(gp)
}



# A wrraper for clustering genes by gaussian mixed modeling, PAM or hierarchical (not used currently)
cluster.by.genes <- function(normalized.dat, gene.set = NULL, class.num = NULL, method = c("GMM", "PAM","HC")){
  method <- match.arg(method)
  if(method == "GMM")
    classifier <- cluster.GMM
  if(method == "PAM")
    classifier <- cluster.PAM
  if(method == "HC")
    classifier <- cluster.hierarchical
  
  class.call <- classifier(normalized.dat = normalized.dat, gene.set = gene.set,class.num = class.num)
  
  return(class.call)
}

# PAM clustering function (not used currently)
cluster.PAM <- function(normalized.dat, gene.set= NULL, class.num = NULL){
  require(cluster)
  pam.fast <- function(x,k) list(cluster = pam(x,k, cluster.only=TRUE))
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  
  if(is.null(class.num)){
    gs <- clusGap(normalized.dat[,gene.set,drop=F], pam.fast, K.max = 5, B = 100, verbose = F)
    class.num <- maxSE(f = gs$Tab[,"gap"],SE.f = gs$Tab[,"SE.sim"],method = "Tibs2001SEmax")
    
  }
  class.call <- as.numeric(factor(unlist(pam.fast(x = normalized.dat[,gene.set,drop=F],k = class.num))))
  return(class.call)
}

# Gaussian mixed model clustering function (not used currently)
cluster.GMM <- function(normalized.dat, gene.set = NULL, class.num = NULL){
  require(mclust)
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  
  class.calls <- Mclust(normalized.dat[,gene.set,drop=F],G = class.num)$classification
  return(class.calls)
  
}

# hierarchical clustering function (not used currently)
cluster.hierarchical <- function(normalized.dat, gene.set = NULL, class.num = NULL){
  require(cluster)
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  
  if(is.null(class.num)){
    hc <- function(x,k) list(cluster = cutree(hclust(dist(x = x),method = "ave"),k = k))
    gs <- clusGap(normalized.dat[,gene.set,drop=F], hc, K.max = 5, B = 100, verbose = F)
    class.num <- maxSE(f = gs$Tab[,"gap"],SE.f = gs$Tab[,"SE.sim"],method = "Tibs2001SEmax")
    
  }
  
  hcav <- hclust(dist(normalized.dat[,gene.set]),method = "ave")
  class.call <- cutree(tree = hcav,k = class.num)
  return(class.call)
}


# This funciton generates PCA biplots using ggbiplot
pca.biplot <- function(normalized.dat,gene.set = NULL, smp.annot = NULL, smp.annot.name = NULL){
  
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  
  pcobj <- prcomp(x = normalized.dat[,gene.set],scale. = T,center = T)
  
  var.axes <- length(gene.set<=10)
  g1 <- ggbiplot(pcobj = pcobj,choices = 1:2,groups = smp.annot,ellipse = T,circle = F,alpha = 1,var.axes = var.axes,var.scale = 1)
  g1 <- g1 + ggtitle("PCA: samples on 1st and 2nd PC plane")
  g1 <- g1 + theme_bw()
  
  g2 <- ggbiplot(pcobj = pcobj,choices = c(1,3),groups = smp.annot,ellipse = T,circle = F,alpha = 1,var.axes = var.axes,var.scale = 1)
  g2 <- g2 + ggtitle("PCA: samples on 1st and 3rd PC plane")
  g2 <- g2 + theme_bw()
  
  g3 <- ggbiplot(pcobj = pcobj,choices = c(2,3),groups = smp.annot,ellipse = T,circle = F,alpha = 1,var.axes = var.axes,var.scale = 1)
  g3 <- g3 + ggtitle("PCA: samples on 2nd and 3rd PC plane")
  g3 <- g3 + theme_bw()
  
  # Change the title of the legend
  #-------------------------------
  if(!is.null(smp.annot.name)){
    g1 <- g1 + labs(color=smp.annot.name)
    g2 <- g2 + labs(color=smp.annot.name)
    g3 <- g3 + labs(color=smp.annot.name)
  }
  
  
  #g <- grid.arrange(g1,g2,g3,ncol=3)
  return(list(pc12=g1,pc13=g2,pc23=g3))
  
}


# This is the GGally rendition of parallel coordinate plots (not used currently)
parcor.plot1 <- function(normalized.dat, gene.set = NULL, smp.annot){
  require(GGally)
  #require(gridExtra)
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  groups <- smp.annot
  if(is.numeric(groups))
    groups <- factor(paste("Group",smp.annot))
  
  d <- data.frame(groups = groups,normalized.dat[,gene.set])
  #   d.mean <- aggregate(formula = .~smp.annot,FUN = mean,data = d)
  #   g.all <- ggparcoord(data = d,columns = 2:ncol(d),groupColumn = "smp.annot",scale = "uniminmax",alphaLines = .5,mapping = aes(size = 1 ))
  #   g.mean <- ggparcoord(data = d.mean,columns = 2:ncol(d),groupColumn = "smp.annot",scale = "globalminmax",alphaLines = .5,mapping = aes(size = 1 ))
  #   g <- grid.arrange(g.all,g.mean,ncol=2)
  g <- ggparcoord(data = d,columns = 2:ncol(d),groupColumn = "groups",scale = "std",alphaLines = 1,mapping = aes(size = 1 ))
  g <- g + scale_x_discrete(name="")
  return(g)
}


# This funciton generates parallel coordinate plots as well as the average trend
parcor.plot2 <- function(normalized.dat, gene.set = NULL, smp.annot, smp.annot.name,scale.by.gene = T){
  require(reshape)
  
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(is.null(gene.set))
    gene.set <- colnames(normalized.dat)
  
  if(any(!gene.set %in% colnames(normalized.dat)))
    stop("all gene.set needs to be in colnames(normalized.dat)")
  
  if(length(unique(smp.annot))<2)
    stop("There should be at least two smaple categories in smp.annot")
  
  mat <- normalized.dat[,gene.set]
  if(scale.by.gene)
    mat <- scale(mat)
  
  ## if scale by group
  # for(grp in levels(groups)){
  #   mat[groups==grp,] <- scale(mat[groups==grp,])
  # }
  
  
  groups <- smp.annot
  if(is.numeric(groups))
    groups <- factor(paste("Group",smp.annot))
  
  # Make the dataframe and convert to long
  df <- data.frame(Sample.ID=rownames(mat),groups = groups, mat)
  long.dat <- melt(df, id.vars  = c("Sample.ID","groups"))
  tmp <- colnames(long.dat);tmp[tmp=="variable"] <- "gene"; tmp[tmp=="value"]<- "Expression"
  colnames(long.dat) <- tmp
  
  # Make and add order (Time)
  ord <- as.numeric(factor(long.dat$gene))
  long.dat$ord <- ord
  
  # Generate the ggplot object
  p <-  ggplot(data = long.dat, aes(x = ord, y = Expression, group = Sample.ID)) 
  p <- p + geom_line(alpha=0.2)
  p <- p + stat_smooth(aes(group = 1))
  p <- p + stat_summary(aes(group = 1),geom = "point", fun.y = mean, shape = 17, size = 3)
  p <- p + facet_grid(groups ~ .) 
  p <- p + scale_x_discrete(name="", limits= levels(long.dat$gene))
  p <- p + ggtitle(paste("Parallel coordinate plot by",smp.annot.name))
  p <- p + theme(axis.text.x  = element_text(angle=60, vjust=0.5, size=15))
  p <- p + theme_bw()
  
  return(p)
}

# This function generates trend plots
gene.trend.plot <- function(normalized.dat,trend.ids, t.obs, gene, smp.subset = NULL,subset.variable = NULL,adj.to.tmin=T, xlab = NULL){
  require(ggplot2)
  #require(mgcv)
  if(!is.matrix(normalized.dat))
    stop("normalized.dat needs to be a matrix")
  
  if(!gene %in% colnames(normalized.dat))
    stop("gene need to be in colnames(normalized.dat)")
  
  if(!class(t.obs) %in% c("numeric","integer","factor","character")| length(t.obs)!=nrow(normalized.dat))
    stop("t.obs needs to be a numeric, integer, factor, or character vector of the same length as the row.num of normalized.dat")
  
  t.is.numeric <- is.numeric(t.obs)
  # This is to accomodate and label character/factor for x-axis in trend plots
  get.t.obs.labs <- function(x){return(x)}
  if(is.factor(t.obs) | is.character(t.obs)){
    #tmp <- as.numeric(sapply(as.list(as.character(t.obs)), function (k)  strsplit(k, "[^[:digit:]]")[[1]][strsplit(k, "[^[:digit:]]")[[1]]!=""]
    t.obs.labs <- levels(as.factor(t.obs))
    t.obs <- as.numeric(factor(t.obs))
    get.t.obs.labs <- function(x){return(t.obs.labs[x])}
  }
  
  
  if(is.null(smp.subset))
    smp.subset <- factor(rep("all obs",nrow(normalized.dat)))
  
  mat <- normalized.dat[,gene,drop=F]
  
  
  groups <- smp.subset
  if(is.numeric(groups))
    groups <- factor(paste("Group",smp.subset))
  
  #cast as factor (any other class hasn't been covered)
  smp.subset <- as.factor(smp.subset)
  
  #Get all levels of t before subsetting
  tlevels <- levels(factor(t.obs))
  
  # Make the dataframe
  df <- data.frame(Trend.ID= trend.ids, t= t.obs, groups = groups, mat)
  colnames(df)[colnames(df)==gene] <- "Expression"
  
  # check no repeated sample given Trend.ID,time point and group
  df$Trend.ID <- as.character(df$Trend.ID)
  repeated <- any(duplicated(interaction(df$Trend.ID,df$t,df$groups)))
  df$Trend.ID <- as.factor(df$Trend.ID)
  
  #collapse by sample ID if repeated
  df0 <- df
  if(repeated & adj.to.tmin)
    df <- aggregate(formula = Expression ~ Trend.ID : t : groups, data = df, FUN = mean)
  
  # check each Trend.ID.i:groups has the same t_min
  all.same.tmin <- length(unique(aggregate(formula = t ~ Trend.ID:groups, data = df, FUN = function(x) min(as.numeric(factor(x = x,levels = tlevels))))$t)) == 1
  
  if(!all.same.tmin)
    #unaggragate
    df <- df0
  
  
  
  # update adjust.to.tmin based on whether on all.same.tmin is true
  adj.to.tmin <- adj.to.tmin & all.same.tmin
  
  # Labeling
  xlab0 <- paste("Order of observations")
  if(!is.null(xlab))
    xlab0 <- paste(xlab0, " (by ",xlab,")",sep="")
  xlab <- xlab0
  #   if(t.is.numeric){
  #     xlab <- "Observation time"
  #     cat("it is numeric\n")
  #   }
  
  
  ylab <- "Expression"
  if(adj.to.tmin)
    ylab <- "t0 subtracted expression"
  
  # Adjust to t0
  if(adj.to.tmin){
    for(smp.i in levels(df$Trend.ID)){
      for(gr in levels(df$groups[df$Trend.ID == smp.i])){
        if(!t.is.numeric){
          df$t[df$Trend.ID == smp.i & df$groups == gr] <- as.numeric(factor(df$t[df$Trend.ID == smp.i & df$groups == gr],levels = tlevels))
          df$Expression[df$Trend.ID == smp.i & df$groups == gr] <- df$Expression[df$Trend.ID == smp.i & df$groups == gr]- df$Expression[df$Trend.ID == smp.i & df$groups == gr & df$t == 1]
        }else{
          df$Expression[df$Trend.ID == smp.i & df$groups == gr] <- df$Expression[df$Trend.ID == smp.i & df$groups == gr]- df$Expression[df$Trend.ID == smp.i & df$groups == gr & df$t == min(df$t)]
        }
      }
    }
  }
  
  
  
  
  
  # Generate the ggplot object
  
  stratification.lab <- NULL
  if(nlevels(df$groups)>1)
    stratification.lab <- paste("stratified by",subset.variable)
  
  if(!repeated){
    p <-  ggplot(data = df, aes(x = t, y = Expression, group = Trend.ID)) 
    p <- p + geom_line(alpha=0.2)
    p <- p + stat_smooth(aes(group = 1),colour ="chartreuse3",size=1.5)
    p <- p + stat_summary(aes(group = 1),geom = "point", fun.y = mean, shape = 19, size = 3)
    p <- p + facet_grid(. ~ groups) 
    p <- p + ggtitle(paste("Expression trend for",gene,stratification.lab)) + xlab(xlab) + ylab(ylab)
    p <- p + scale_x_continuous(label=get.t.obs.labs)
    p <- p + theme_bw()
    return(p)
  }
  
  if(adj.to.tmin){
    p <-  ggplot(data = df, aes(x = t, y = Expression, group = Trend.ID)) 
    p <- p + geom_line(aes(colour = Trend.ID))
    p <- p + facet_grid(. ~ groups)
    p <- p + ggtitle(paste("Average expression trend for",gene,stratification.lab)) + xlab(xlab) + ylab(ylab)
    p <- p + scale_x_continuous(label=get.t.obs.labs)
    p <- p + theme_bw()
    return(p)
  }
  
  p <-  ggplot(data = df, aes(x = t, y = Expression, group = Trend.ID)) 
  p <- p + geom_point(aes(colour = Trend.ID),size = 3)
  #p + stat_smooth(method=ifelse(class(try(loess(Expression~t, df[df$groups=="Group 3",])))=="try-error","glm","loess")....) 
  p <- p + facet_grid(. ~ groups)
  p <- p + stat_smooth(aes(group = Trend.ID, colour = Trend.ID),se=F)
  p <- p + ggtitle(paste("Average expression trend for",gene,stratification.lab)) + xlab(xlab) + ylab(ylab)
  p <- p + scale_x_continuous(label=get.t.obs.labs)  
  p <- p + theme_bw()
  return(p)
}
