#=============================================================================================
# The main functions for generating the results of the SGD module
# Dependencies:
# "SGD_util.R"  sourced by describe.selected.genes
# "source - utils.r" sourced by store.SGD.results
#=============================================================================================

# This is the main function of the SGD.module. It is a wrapper around describe.selected.gene (plot rendering) and store.SGD.results (plot storing)
# functions it returns a list of lists (tree) containing the uri to the plots. The structure of this list of lists encodes the html hierarchy

run.SGD.module <- function(normalized.dat,
                           smp.annots,
                           color.annots,
                           color.cat.annots,
                           gene.names.to.describe,
                           describe.genes.by = NULL,
                           stratify.gene.desription.by = NULL,
                           trend.expression.by = NULL,
                           order.expression.trend.by = NULL,
                           stratify.expression.trend.by = NULL,
                           file.extension = c("png","pdf","tiff","jpg","bmp")){
  SGD <- describe.selected.genes(normalized.dat=normalized.dat,
                                 smp.annots=smp.annots,
                                 color.annots=color.annots,
                                 color.cat.annots=color.cat.annots,
                                 gene.names.to.describe=gene.names.to.describe,
                                 describe.genes.by = describe.genes.by,
                                 stratify.gene.desription.by = stratify.gene.desription.by,
                                 trend.expression.by = trend.expression.by,
                                 order.expression.trend.by = order.expression.trend.by,
                                 stratify.expression.trend.by = stratify.expression.trend.by)
  
  SGD.dir <- store.SGD.results(SGD = SGD,file.extension = file.extension)
  
  return(SGD.dir)  
}

# This function takes in the normalized data, annotations, annotation colors and returns plot objects
describe.selected.genes <- function(normalized.dat,
                                    smp.annots,
                                    color.annots,
                                    color.cat.annots,
                                    gene.names.to.describe,
                                    describe.genes.by = NULL,
                                    stratify.gene.desription.by = NULL, # <---- This is not currently being used
                                    trend.expression.by = NULL,
                                    order.expression.trend.by = NULL,
                                    stratify.expression.trend.by = NULL){
  
  source("SGD_util.R")
  # validate the input
  if(!is.character(gene.names.to.describe))
    stop("gene.names.to.describe needs to be character")
  
  if(any(!gene.names.to.describe %in% colnames(normalized.dat)))
    stop("gene.names.to.describe needs to be in colnames of normalized")
  
  if(!is.data.frame(smp.annots))
    stop("smp.annots need to be a data.frame")
  
  if(sum(!complete.cases(smp.annots))!= 0)
    warning("There are missing annotations: samples with missing annotation will be disregarded")
  
  if(!all.equal(rownames(smp.annots),rownames(normalized.dat)))
    stop("rownames of the smp.annots should match those of the normalized.dat")
  
  if(!is.matrix(color.annots))
    stop("color.annots needs to be a matrix")
  
  if(!all.equal(rownames(color.annots),rownames(normalized.dat)))
    stop("rownames of the color.annots should match those of the normalized.dat")
  
  if(is.null(describe.genes.by))
    describe.genes.by <- "no_annotation"
  
  if(is.null(stratify.expression.trend.by))
    stratify.expression.trend.by <- "no_annotation"
  
  # Trend plot options validation
  generate.trend.plot <- FALSE
  if(!is.null(trend.expression.by) & !is.null(order.expression.trend.by)){
    if(!trend.expression.by %in% colnames(smp.annots))
      stop("trend.expression.by needs to be in the colnames of smp.annots")
    if(!order.expression.trend.by %in% colnames(smp.annots))
      stop("order.expression.trend.by needs to be in the colnames of smp.annots")
    generate.trend.plot <- TRUE
  }
  
  # ---->validation of stratify <------
  
  # Determine description-type
  description.type <- "single.gene"
  if(length(gene.names.to.describe)>1)
    description.type <- "multi.gene"
  
  
  # Initiate plot lists
  #--------------------
  univar.plots <- cor.plots <- PCA.biplots <- parcor.plots <- list()
  length(univar.plots) <- length(cor.plots) <- length(PCA.biplots) <- length(parcor.plots)  <- length(describe.genes.by)
  names(univar.plots) <- names(cor.plots) <- names(PCA.biplots) <- names(parcor.plots)  <- describe.genes.by
  
  trend.plots <- list()
  length(trend.plots) <- length(stratify.expression.trend.by) # currently 1
  names(trend.plots) <- stratify.expression.trend.by
  #single or multi gene shared analysis
  #------------------------------------
  
  for(annot.i in describe.genes.by){
    #print(annot.i)
    smp.annot.i <- smp.annots[[annot.i]]
    #univariate plot: violin or scatter plot 1 per annot
    #----------------------------------------------------
    for(gene.i in gene.names.to.describe){
      univar.plot.fun <- violin.by.gene
      if(is.numeric(smp.annots[[annot.i]]))
        univar.plot.fun <- scatter.by.gene
      
      # update status report
      cat(paste("document.write('<p>Creating univariate plot by",annot.i,"for gene",gene.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      
      univar.plots[[annot.i]][[gene.i]] <- univar.plot.fun(normalized.dat = normalized.dat,
                                                           gene = gene.i,
                                                           smp.annot = smp.annot.i,
                                                           smp.annot.name = annot.i)
    }
    
    #corplot
    #-------
    genes.to.compare.to <- NULL
    if(length(gene.names.to.describe)>1)
      genes.to.compare.to <- gene.names.to.describe[-1]
    
    # trichotomize if smp.annot.i is numeric
    #---------------------------------------
    if(is.numeric(smp.annot.i)){
      tmp <- rep("Low",length(smp.annot.i))
      tmp[smp.annot.i> quantile(smp.annot.i,probs = .25,na.rm = T)]<- "Average"
      tmp[smp.annot.i> quantile(smp.annot.i,probs = .75,na.rm = T)]<- "High"
      tmp[is.na(smp.annot.i)] <- NA
      smp.annot.i <- factor(tmp)
    }
    
    # update status report
    cat(paste("document.write('<p>Creating corrlation plot stratified by",annot.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    cor.plots[[annot.i]][["select_genes_corplot"]] <- corplot.by.gene(normalized.dat = normalized.dat,
                                                                      gene = gene.names.to.describe[1],
                                                                      smp.annot = smp.annot.i,
                                                                      #col.annot = col2hex(color.cat.annots[[annot.i]][as.character(smp.annot.i)]),
                                                                      annot.name = annot.i,
                                                                      genes.to.compare.to = genes.to.compare.to)
  }
  
  # multi-gene analysis
  #--------------------
  if(description.type == "multi.gene"){
    
    for(annot.i in describe.genes.by){
      #print(annot.i)
      smp.annot.i <- smp.annots[[annot.i]]
      
      # trichotomize if smp.annot.i is numeric
      #---------------------------------------
      if(is.numeric(smp.annot.i)){
        tmp <- rep("Low",length(smp.annot.i))
        tmp[smp.annot.i> quantile(smp.annot.i,probs = .25,na.rm = T)]<- "Average"
        tmp[smp.annot.i> quantile(smp.annot.i,probs = .75,na.rm = T)]<- "High"
        tmp[is.na(smp.annot.i)] <- NA
        smp.annot.i <- factor(tmp)
      }
      
      
      #PCA.biplots
      #-----------
      if(length(gene.names.to.describe)>4){
        
        # update status report
        cat(paste("document.write('<p>Creating PCA biplots labeled by",annot.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        
        PCA.biplots[[annot.i]] <- pca.biplot(normalized.dat = normalized.dat,gene.set = gene.names.to.describe,smp.annot = smp.annot.i,smp.annot.name = annot.i)
      }
      
      
      #parcor.plots
      #--------------
      if(nlevels(smp.annot.i)>1){
        # update status report
        cat(paste("document.write('<p>Creating parallel coordinate plots for",annot.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        parcor.plots[[annot.i]] <- parcor.plot2(normalized.dat = normalized.dat,gene.set = gene.names.to.describe,smp.annot = smp.annot.i,smp.annot.name = annot.i)
      }else{
        warning(paste(annot.i,"less than 2 levels when casted as a categorical variable. Therefore, parallel coordinate plot is not generated"))
      }
    }
    
  }
  
  # Trend analysis
  #---------------
  if(generate.trend.plot){
    for (gene.i in gene.names.to.describe){
      #print(gene.i)
      
      # update status report
      cat(paste("document.write('<p>Creating trend plots for",gene.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      
      trend.plots[[stratify.expression.trend.by]][[gene.i]] <- gene.trend.plot(normalized.dat = normalized.dat,
                                                                               trend.ids = smp.annots[,trend.expression.by],
                                                                               t.obs = smp.annots[,order.expression.trend.by],
                                                                               gene = gene.i,
                                                                               smp.subset = smp.annots[[stratify.expression.trend.by]],
                                                                               subset.variable = stratify.expression.trend.by,
                                                                               adj.to.tmin = T,
                                                                               xlab = order.expression.trend.by)
      
    }
    
  }
  
  
  # build the Select Genes Descriptives object
  #-------------------------------------------
  SGD <- list(univar.plots = univar.plots,
              cor.plots = cor.plots,
              PCA.biplots = PCA.biplots,
              parcor.plots = parcor.plots,
              trend.plots = trend.plots)
  
  return(SGD)
}


# This function handles storing the Select Gene Descriptives
store.SGD.results <- function(SGD, file.extension = c("png","pdf","tiff","jpg","bmp")){
  file.extension <- match.arg(file.extension)
  source("source - utils.r")
  
  plt.directories <- SGD # initiate plt.directories
  
  SGD.path <- "./results/SGD/"
  dir.create(SGD.path,showWarnings = F)
  for(plot.type.i in names(SGD)){
    #cat(plot.type.i," ")
    plttype.level.path.i <- correct.filename(file.path(SGD.path,plot.type.i))
    dir.create(plttype.level.path.i,showWarnings = F)
    for(smp.annot.i in names(SGD[[plot.type.i]])){
      #cat(smp.annot.i,"\n")
      annot.level.path.i <- correct.filename(file.path(plttype.level.path.i, smp.annot.i))
      
      if(all(class(SGD[[plot.type.i]][[smp.annot.i]]) == "list")){
        dir.create(annot.level.path.i,showWarnings = F)
        for(plt.i in names(SGD[[plot.type.i]][[smp.annot.i]])){
          
          if(!is.null(SGD[[plot.type.i]][[smp.annot.i]][[plt.i]])){
            filename <- correct.filename(file.path(annot.level.path.i,plt.i))
            print(filename)
            #           tmp <- drawplot(filename=filename)
            #           SGD[[plot.type.i]][[smp.annot.i]][[plt.i]]
            #           dev.off()
            if("ggplot" %in% class(SGD[[plot.type.i]][[smp.annot.i]][[plt.i]])){
              ggsave(filename = paste(filename,file.extension,sep="."), plot = SGD[[plot.type.i]][[smp.annot.i]][[plt.i]],width = 12,height = 8,scale = 1,dpi = 300,units = "in")
            }else{
              drawplot(filename = filename,type = file.extension,width = 2,height = 2)
              print(SGD[[plot.type.i]][[smp.annot.i]][[plt.i]])
              dev.off()
            }
            #store the directory for the plot
            plt.directories[[plot.type.i]][[smp.annot.i]][[plt.i]] <- paste(filename,file.extension,sep=".")
          }
        }
      }else{
        if(!is.null(SGD[[plot.type.i]][[smp.annot.i]])){
          filename <- annot.level.path.i
          print(filename)
          #         tmp <- drawplot(filename=filename)
          #         SGD[[plot.type.i]][[smp.annot.i]]
          #         dev.off()
          if("ggplot" %in% class(SGD[[plot.type.i]][[smp.annot.i]])){
            ggsave(filename = paste(filename,file.extension,sep="."), plot = SGD[[plot.type.i]][[smp.annot.i]],,width = 12,height = 8,scale = 1,dpi = 300,units = "in" )
          }else{
            drawplot(filename = filename,type = file.extension,width = 2,height = 2)
            print(SGD[[plot.type.i]][[smp.annot.i]])
            dev.off()
          }
          plt.directories[[plot.type.i]][[smp.annot.i]] <- paste(filename,file.extension,sep=".")
        }
        
      }
    }
  }
  return(plt.directories)
}
